package application

import (
	"fmt"
	"github.com/google/uuid"

	"net/http"
	"path"
	"sirlab.de/go/knowyt/game"
	"sirlab.de/go/knowyt/user"
	"strconv"
)

func (app *Application) SavePlayer(usr *user.User, w http.ResponseWriter, r *http.Request) {
	gameRef := r.URL.Query().Get("g")
	gm, err := app.GetGameById(gameRef)
	if err != nil {
		w.WriteHeader(http.StatusNotFound)
		fmt.Fprintf(w, "game not found")
		return
	}

	if usr.GetGameId() != gameRef || !usr.IsGamemaster() {
		w.WriteHeader(http.StatusForbidden)
		fmt.Fprintf(w, "forbidden")
		return
	}

	id := r.URL.Query().Get("id")
	name := r.URL.Query().Get("name")
	score := r.URL.Query().Get("score")
	authcode := r.URL.Query().Get("authcode")
	if name == "" {
		w.WriteHeader(http.StatusBadRequest)
		fmt.Printf("player name must not be empty\n")
		fmt.Fprintf(w, "server error")
		return
	}
	if id != "" {
		if err := app.modifyUser(id, gm, name, authcode); err != nil {
			w.WriteHeader(http.StatusInternalServerError)
			fmt.Printf("%v\n", err)
			fmt.Fprintf(w, "server error")
			return
		}
	} else {
		if err := app.createUser(gm, name, authcode); err != nil {
			w.WriteHeader(http.StatusInternalServerError)
			fmt.Printf("%v\n", err)
			fmt.Fprintf(w, "server error")
			return
		}
	}

	if err := app.updateScore(gm, id, score); err != nil {
		w.WriteHeader(http.StatusInternalServerError)
		fmt.Printf("%v\n", err)
		fmt.Fprintf(w, "server error")
		return
	}

	fmt.Fprintf(w, "ok")
}

func (app *Application) updateScore(gm *game.Game, id string, scoreString string) error {
	scoreInt, err := strconv.Atoi(scoreString)
	if err != nil {
		return nil
	}

	if err := gm.SetScoreByPlayer(id, scoreInt); err != nil {
		return err
	}
	if err := app.saveGameState(gm); err != nil {
		return err
	}
	return nil
}

func (app *Application) modifyUser(id string, gm *game.Game, name, authcode string) error {
	if authcode != "" {
		if usr, err := app.GetUserByAuthcode(authcode); err == nil && usr.GetId() != id {
			return fmt.Errorf("%s authcode already in use", gm.GetId())
		}
	}

	modifyUser := app.users[id]
	if modifyUser.GetId() != id || modifyUser.GetGameId() != gm.GetId() {
		return fmt.Errorf("%s couldn't find player %s in game", gm.GetId(), id)
	}
	modifyUser.SetName(name)
	modifyUser.SetAuthcode(authcode)
	if err := modifyUser.SaveUser(); err != nil {
		return err
	}
	gm.UpdatePlayer(modifyUser)
	return nil
}

func (app *Application) createUser(gm *game.Game, name, authcode string) error {
	if _, err := app.GetUserByAuthcode(authcode); err == nil {
		return fmt.Errorf("%s authcode already in use", gm.GetId())
	}

	dirName := path.Join(app.config.DataDir, "users")
	userNewUuid := uuid.NewString()
	fileName := path.Join(dirName, userNewUuid+".json")
	newUser := user.CreateUser(fileName, gm.GetId())
	newUser.SetRole(user.ROLE_PLAYER)
	newUser.SetName(name)
	newUser.SetAuthcode(authcode)
	if err := newUser.SaveUser(); err != nil {
		return err
	}
	app.users[newUser.GetId()] = newUser
	gm.AddPlayer(newUser)
	return nil
}
